@@ -1,6 +1,6 @@
 ;;; tmr.el --- Set timers using a convenient notation -*- lexical-binding: t -*-
 
-;; Copyright (C) 2020-2025  Free Software Foundation, Inc.
+;; Copyright (C) 2020-2026  Free Software Foundation, Inc.
 
 ;; Author: Protesilaos Stavrou <info@protesilaos.com>,
 ;;         Damien Cassou <damien@cassou.me>,
@@ -8,7 +8,7 @@
 ;;         Steven Allen <steven@stebalien.com>
 ;; Maintainer: Protesilaos Stavrou <info@protesilaos.com>
 ;; URL: https://github.com/protesilaos/tmr
-;; Version: 1.2.1
+;; Version: 1.3.0
 ;; Package-Requires: ((emacs "29.1"))
 ;; Keywords: convenience, timer
 
@@ -120,11 +120,42 @@ Each function must accept a timer as argument."
 Each function must accept a timer as argument."
   :type 'hook)
 
+(defcustom tmr-timer-paused-functions
+  (list #'tmr-print-message-for-paused-timer)
+  "Functions to execute when a timer is paused.
+Each function must accept a timer as argument."
+  :package-version '(tmr . "1.3.0")
+  :type 'hook)
+
+(defcustom tmr-timer-resumed-functions
+  (list #'tmr-print-message-for-resumed-timer)
+  "Functions to execute when a timer is resumed.
+Each function must accept a timer as argument."
+  :package-version '(tmr . "1.3.0")
+  :type 'hook)
+
 (defcustom tmr-finished-indicator "✔"
   "Indicator for a finished timer, shown in `tmr-tabulated-view'."
   :package-version '(tmr . "1.0.0")
   :type 'string)
 
+(defcustom tmr-acknowledge-timer-text "ack"
+  "Text to confirm that a timer is acknowledged.
+This is the input to be given to the minibuffer prompt that asks for
+confirmation when a timer has to be acknowledged (acknowledgement here
+means that the user confirms that the timer has elapsed)."
+  :package-version '(tmr . "1.3.0")
+  :type 'string)
+
+(defcustom tmr-tabulated-refresh-interval 5
+  "Refresh the `tmr-tabulated-view' buffer after these many seconds.
+If the value is nil, then never automatically refresh that buffer: the
+user can do that manually by invoking the command `revert-buffer'."
+  :package-version '(tmr . "1.3.0")
+  :type '(choice
+          (natnum :tag "Seconds to auto-refresh the `tmr-tabulated-view' buffer")
+          (const :tag "Never auto-refresh the `tmr-tabulated-view' buffer" nil)))
+
 (defun tmr-select-and-resize (window)
   "Select WINDOW and fit it to its buffer."
   (select-window window)
@@ -228,10 +259,15 @@ Longer descriptions will be truncated."
   :group 'tmr-faces)
 
 (defface tmr-finished '((t :inherit error))
-  "Face for styling the confirmation of a finished timer."
+  "Face for styling the description of a finished timer."
   :package-version '(tmr . "1.0.0")
   :group 'tmr-faces)
 
+(defface tmr-paused '((t :inherit bold))
+  "Face for styling the description of a paused timer."
+  :package-version '(tmr . "1.3.0")
+  :group 'tmr-faces)
+
 (defface tmr-tabulated-start-time
   '((((class color) (min-colors 88) (background light))
      :foreground "#004476")
@@ -262,6 +298,11 @@ Longer descriptions will be truncated."
   :package-version '(tmr . "1.1.0")
   :group 'tmr-faces)
 
+(defface tmr-tabulated-paused '((t :inherit bold))
+  "Face for styling the description of a paused timer."
+  :package-version '(tmr . "1.3.0")
+  :group 'tmr-faces)
+
 (defface tmr-tabulated-acknowledgement
   '((t :inherit bold))
   "Acknowledgement indicator in the `tmr-tabulated-view'."
@@ -303,7 +344,6 @@ Longer descriptions will be truncated."
    :documentation "Time at which the timer was created.")
   (end-date
    nil
-   :read-only t
    :documentation "Time at which the timer finishes.")
   (finishedp
    nil
@@ -324,7 +364,11 @@ Longer descriptions will be truncated."
   (description
    nil
    :read-only nil
-   :documentation "Optional string describing the purpose of the timer, e.g., \"Stop the oven\"."))
+   :documentation "Optional string describing the purpose of the timer, e.g., \"Stop the oven\".")
+  (paused-remaining
+   nil
+   :read-only nil
+   :documentation "Remaining seconds when the timer was paused or nil if not paused."))
 
 (defun tmr--long-description (timer)
   "Return a human-readable description for TIMER."
@@ -351,6 +395,8 @@ Longer descriptions will be truncated."
               (concat "; " (propertize "acknowledge" 'face 'tmr-must-be-acknowledged)))
              ((tmr--timer-finishedp timer)
               (concat "; " (propertize "finished" 'face 'tmr-finished)))
+             ((when-let* ((remaining (tmr--timer-paused-remaining timer)))
+              (format "; %s: remaining %s" (propertize "PAUSED" 'face 'tmr-paused) (tmr--format-seconds remaining))))
              (t ""))
             (if description
                 (concat "; " (propertize description 'face 'tmr-description))
@@ -392,24 +438,30 @@ optional `tmr--timer-description'."
 
 (defun tmr--format-remaining (timer)
   "Format remaining time of TIMER."
-  (if (tmr--timer-finishedp timer)
-      tmr-finished-indicator
+  (cond
+   ((tmr--timer-finishedp timer)
+    tmr-finished-indicator)
+   ((when-let* ((remaining (tmr--timer-paused-remaining timer)))
+      (tmr--format-seconds remaining)))
+   (t
     (let* ((seconds (tmr--get-seconds timer))
            (str (tmr--format-seconds seconds)))
       (if (< seconds 0)
           ;; Negative remaining time occurs for non-acknowledged timers with
           ;; additional duration.
           (propertize str 'face 'tmr-must-be-acknowledged)
-        str))))
+        str)))))
 
 (defun tmr--format-duration (timer)
   "Format duration of TIMER."
   (let ((input (tmr--timer-input timer)))
-    (if (string-match-p ":" input)
-        (tmr--format-seconds (tmr--get-seconds timer))
-      (if (string-match-p "\\`[0-9]+\\(?:\\.[0-9]+\\)?\\'" input)
-          (concat input "m")
-        input))))
+    (cond
+     ((string-match-p ":" input)
+      (tmr--format-seconds (tmr--get-seconds timer)))
+     ((string-match-p "\\`[0-9]+\\(?:\\.[0-9]+\\)?\\'" input)
+      (concat input "m"))
+     (t
+      input))))
 
 (defun tmr--format-time (time)
   "Return a human-readable string representing TIME."
@@ -441,8 +493,7 @@ optional `tmr--timer-description'."
      (t (user-error "TMR Made Ridiculous; append character for [m]inutes, [h]ours, [s]econds")))))
 
 (defvar tmr--timers nil
-  "List of timer objects.
-Populated by `tmr' and then operated on by `tmr-cancel'.")
+  "List of timer objects.")
 
 (defvar tmr--update-hook nil
   "Hooks to execute when timers are changed.")
@@ -485,6 +536,24 @@ cancelling the original one."
   (setf (tmr--timer-description timer) description)
   (run-hooks 'tmr--update-hook))
 
+(defun tmr-toggle-pause (timer)
+  "Toggle pause/resume state of TIMER."
+  (interactive (list (tmr--read-timer "Pause/resume timer: " :active)))
+  (if-let* ((remaining (tmr--timer-paused-remaining timer)))
+      (progn
+        (setf (tmr--timer-end-date timer) (time-add (current-time) remaining))
+        (setf (tmr--timer-timer-object timer) (run-with-timer remaining nil #'tmr--complete timer))
+        (setf (tmr--timer-paused-remaining timer) nil)
+        (run-hooks 'tmr--update-hook)
+        (run-hook-with-args 'tmr-timer-resumed-functions timer))
+    (let ((remaining (tmr--get-seconds timer)))
+      (when (> remaining 0)
+        (cancel-timer (tmr--timer-timer-object timer))
+        (setf (tmr--timer-paused-remaining timer) remaining)
+        (run-hooks 'tmr--update-hook)
+        (run-hook-with-args 'tmr-timer-paused-functions timer)))))
+
+
 (defun tmr-toggle-acknowledge (timer)
   "Toggle ackowledge flag of TIMER."
   (interactive
@@ -545,6 +614,7 @@ If there are no timers, throw an error."
 (declare-function android-notifications-notify "androidselect.c" (&rest params))
 (declare-function w32-notification-notify "w32fns.c" (&rest params))
 (declare-function haiku-notifications-notify "haikuselect.c" (&rest params))
+(defvar notifications-application-icon)
 
 (defun tmr-notification-notify (timer)
   "Dispatch a notification for TIMER.
@@ -578,7 +648,7 @@ Read Info node `(elisp) Desktop Notifications' for details."
                 :title title
                 :body body
                 :app-name "GNU Emacs"
-                :app-icon 'emacs
+                :app-icon notifications-application-icon
                 :urgency tmr-notification-urgency
                 :sound-file tmr-sound-file))))
     (display-warning 'tmr "Emacs has no DBUS support, TMR notifications unavailable")))
@@ -606,13 +676,21 @@ Read Info node `(elisp) Desktop Notifications' for details."
   "Show a `message' informing the user that TIMER is cancelled."
   (message "Cancelled: <<%s>>" (tmr--long-description timer)))
 
+(defun tmr-print-message-for-paused-timer (timer)
+  "Show a `message' informing the user that TIMER is paused."
+  (message "Paused: <<%s>> (REMAINING %s)" (tmr--long-description timer) (tmr--format-remaining timer)))
+
+(defun tmr-print-message-for-resumed-timer (timer)
+  "Show a `message' informing the user that TIMER is resumed."
+  (message "Resumed: <<%s>> (REMAINING %s)" (tmr--long-description timer) (tmr--format-remaining timer)))
+
 (defvar tmr-duration-history nil
   "Minibuffer history of `tmr' durations.")
 
 (defun tmr--read-duration (&optional default)
   "Ask the user to type a duration.
 If DEFAULT is provided, use that as a default."
-  (let ((def (or default (nth 0 tmr-duration-history))))
+  (let ((def (or default (car tmr-duration-history))))
     (read-string
      (format-prompt
       "N minutes for timer (append `h' or `s' for other units)" def)
@@ -657,9 +735,10 @@ If optional DEFAULT is provided use it as a default candidate."
     (while
         (let ((input
                (read-from-minibuffer
-                (concat (tmr--long-description-for-finished-timer timer)
-                        "\nAcknowledge with `ack' or additional duration: "))))
-          (not (or (equal input "ack")
+                (format "%s\nAcknowledge with `%s' or additional duration: "
+                        (tmr--long-description-for-finished-timer timer)
+                        tmr-acknowledge-timer-text))))
+          (not (or (equal input tmr-acknowledge-timer-text)
                    (when-let* ((duration
                                 (ignore-errors
                                   (tmr--parse-duration (current-time) input))))
@@ -775,7 +854,6 @@ ANNOTATION is an annotation function."
                    (category . ,category))
       (complete-with-action action candidates str pred))))
 
-
 ;;;; Key bindings
 
 (defvar-keymap tmr-prefix-map
@@ -786,8 +864,8 @@ This map should be bound to a global prefix key."
   "t" #'tmr
   "T" #'tmr-with-details
   "l" #'tmr-tabulated-view
-  "c" #'tmr-clone
   "s" #'tmr-reschedule
+  "P" #'tmr-toggle-pause
   "a" #'tmr-toggle-acknowledge
   "e" #'tmr-edit-description
   "r" #'tmr-remove
@@ -809,7 +887,8 @@ This map should be bound to a global prefix key."
   "c" #'tmr-clone
   "a" #'tmr-toggle-acknowledge
   "e" #'tmr-edit-description
-  "s" #'tmr-reschedule)
+  "s" #'tmr-reschedule
+  "P" #'tmr-toggle-pause)
 
 ;;;;; Integration with the `embark' package
 
@@ -821,7 +900,8 @@ This map should be bound to a global prefix key."
   "c" #'tmr-clone
   "a" #'tmr-toggle-acknowledge
   "e" #'tmr-edit-description
-  "s" #'tmr-reschedule)
+  "s" #'tmr-reschedule
+  "p" #'tmr-toggle-pause)
 
 (defvar embark-keymap-alist)
 (defvar embark-post-action-hooks)
@@ -865,6 +945,7 @@ they are set to reasonable default values."
     (propertize (tmr--format-end-date timer) 'face 'tmr-tabulated-end-time)
     (propertize (tmr--format-duration timer) 'face 'tmr-duration)
     (propertize (tmr--format-remaining timer) 'face 'tmr-tabulated-remaining-time)
+    (propertize (if (tmr--timer-paused-remaining timer) "Yes" "") 'face 'tmr-paused)
     (propertize (if (tmr--timer-acknowledgep timer) "Yes" "") 'face 'tmr-tabulated-acknowledgement)
     (propertize (or (tmr--timer-description timer) "") 'face 'tmr-tabulated-description))))
 
@@ -891,17 +972,15 @@ they are set to reasonable default values."
                                 (tabulated-list-print t)
                                 (when end (goto-char (point-max))))
                               ;; HACK: For some reason the hl-line highlighting gets lost here
-                              (when (and (bound-and-true-p global-hl-line-mode)
-                                         (fboundp 'global-hl-line-highlight))
-                                (global-hl-line-highlight))
                               (when (and (bound-and-true-p hl-line-mode)
                                          (fboundp 'hl-line-highlight))
                                 (hl-line-highlight)))
                           (cancel-timer timer)
                           (setq tmr-tabulated--refresh-timer nil)))
                     (cancel-timer timer)))))
-          (setq timer (run-at-time 1 1 refresh)
-                tmr-tabulated--refresh-timer timer)))
+          (when (natnump tmr-tabulated-refresh-interval)
+            (setq timer (run-at-time nil tmr-tabulated-refresh-interval refresh)
+                  tmr-tabulated--refresh-timer timer))))
     (when tmr-tabulated--refresh-timer
       (cancel-timer tmr-tabulated--refresh-timer)
       (setq tmr-tabulated--refresh-timer nil))))
@@ -918,6 +997,7 @@ they are set to reasonable default values."
                ("End" 10 t)
                ("Duration" 10 t)
                ("Remaining" 10 tmr-tabulated--compare-remaining)
+               ("Paused?" 8 t)
                ("Acknowledge?" 14 t)
                ("Description" 0 t)])
   (add-hook 'window-configuration-change-hook #'tmr-tabulated--window-hook nil t)
@@ -955,24 +1035,26 @@ they are set to reasonable default values."
 
 (defun tmr-mode-line--format-remaining (timer)
   "Format remaining time for TIMER with appropriate face."
-  (let* ((secs (float-time (time-subtract (tmr--timer-end-date timer) nil)))
-         (face (cond ((and (< secs 5) (= (% (truncate secs) 2) 0))
-                      '(tmr-mode-line-urgent (:inverse-video t)))
-                     ((< secs 30) 'tmr-mode-line-urgent)
-                     ((= (truncate secs) 30)
-                      '(tmr-mode-line-urgent (:inverse-video t)))
-                     ((= (truncate secs) 60)
-                      '(tmr-mode-line-soon (:inverse-video t)))
-                     ((< secs 120) 'tmr-mode-line-soon)
-                     ((= (truncate secs) 120)
-                      '(tmr-mode-line-soon (:inverse-video t)))
-                     (t 'tmr-mode-line-active)))
-         (formatted (format-seconds
-                     (cond ((< secs 120) "%mm %ss%z")
-                           ((< secs (* 24 60 60)) "%hh %mm%z")
-                           (t "%dd %hh%z"))
-                     secs)))
-    (propertize formatted 'face face)))
+  (if-let* ((remaining (tmr--timer-paused-remaining timer)))
+      (propertize (format "PAUSED %s" (tmr--format-seconds remaining)) 'face 'tmr-paused)
+    (let* ((secs (float-time (time-subtract (tmr--timer-end-date timer) nil)))
+           (face (cond ((and (< secs 5) (= (% (truncate secs) 2) 0))
+                        '(tmr-mode-line-urgent (:inverse-video t)))
+                       ((< secs 30) 'tmr-mode-line-urgent)
+                       ((= (truncate secs) 30)
+                        '(tmr-mode-line-urgent (:inverse-video t)))
+                       ((= (truncate secs) 60)
+                        '(tmr-mode-line-soon (:inverse-video t)))
+                       ((< secs 120) 'tmr-mode-line-soon)
+                       ((= (truncate secs) 120)
+                        '(tmr-mode-line-soon (:inverse-video t)))
+                       (t 'tmr-mode-line-active)))
+           (formatted (format-seconds
+                       (cond ((< secs 120) "%mm %ss%z")
+                             ((< secs (* 24 60 60)) "%hh %mm%z")
+                             (t "%dd %hh%z"))
+                       secs)))
+      (propertize formatted 'face face))))
 
 (defun tmr-mode-line--format-description (timer)
   "Format description for TIMER, truncating if necessary."
